﻿using System;
using System.Resources;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.Printing;
using Infragistics.Win.UltraWinEditors;
using Infragistics.Win.UltraWinGanttView;
using Infragistics.Win.UltraWinSchedule;

namespace PrintGanttView
{
    public partial class PrintGanttView : Form
    {
        #region Private Members
        readonly ResourceManager _rm = Properties.Resources.ResourceManager;
        // Create a new Project other than the Unassigned Project
        private Project _sampleProject;
        #endregion 

        #region Default constructor
        public PrintGanttView()
        {
            InitializeComponent();
            Load += Form1Load;
            btnPrintPreview.Click += btnPrintPreview_Click;
            cmbShowLegend.ValueChanged += cmbShowLegend_ValueChanged;
            editDetailsAreaWidth.ValueChanged += editDetailsAreaWidth_ValueChanged;
            cmbLegendAppearances.ValueChanged += cmbLegendAppearances_ValueChanged;
            cmbAutoSize.ValueChanged += cmbAutoSize_ValueChanged;
            cmbPrintColorStyle.ValueChanged += cmbPrintColorStyle_ValueChanged;
            chkExpandTasksonPrint.CheckedChanged += chkExpandTasksonPrint_CheckedChanged;
            cmbTextCenter.ValueChanged += cmbTextCenter_ValueChanged;
            cmbTextRight.ValueChanged += cmbTextRight_ValueChanged;
            cmbTimelineAutoSize.ValueChanged += cmbTimelineAutoSize_ValueChanged;
        }
        #endregion // Default constructor

        #region Events

        #region Form1Load
        private void Form1Load(object sender, EventArgs e)
        {
            InitializeForm();
        }
        #endregion // Form1Load

        #region btnPrintPreview_Click
        private void btnPrintPreview_Click(object sender, EventArgs e)
        {
            ultraPrintPreviewDialog1.ShowDialog(this);
        }
        #endregion // btnPrintPreview_Click

        #region cmbShowLegend_ValueChanged
        private void cmbShowLegend_ValueChanged(object sender, EventArgs e)
        {
            var currentItem = (UltraComboEditor)sender;
            ultraGanttViewPrintDocument1.LegendSettings.ShowLegend = (ShowLegend)currentItem.Value;
        }
        #endregion // cmbShowLegend_ValueChanged

        #region editDetailsAreaWidth_ValueChanged
        private void editDetailsAreaWidth_ValueChanged(object sender, EventArgs e)
        {
            var numEditor = (UltraNumericEditor)sender;
            ultraGanttViewPrintDocument1.LegendSettings.DetailsAreaWidth = (int)numEditor.Value;
        }
        #endregion // editDetailsAreaWidth_ValueChanged

        #region cmbLegendAppearances_ValueChanged
        private void cmbLegendAppearances_ValueChanged(object sender, EventArgs e)
        {
            string description;
            var combo = (UltraComboEditor)sender;

            // Retrieve the Appearance object based on the selected index, and set it to the PropertyGrid
            pgLegendAppearances.SelectedObject = RetrieveAppearanceBasedOnIndex(combo.SelectedIndex, out description);

            // Update the Appearance Description label
            lblLegendAppearance.Text = description;
        }
        #endregion // cmbLegendAppearances_ValueChanged

        #region cmbAutoSize_ValueChanged
        private void cmbAutoSize_ValueChanged(object sender, EventArgs e)
        {
            var currentItem = (UltraComboEditor)sender;
            ultraGanttViewPrintDocument1.AutoSizeGrid = (AutoSizeGrid)currentItem.Value;
        }
        #endregion // cmbAutoSize_ValueChanged

        #region cmbPrintColorStyle_ValueChanged
        private void cmbPrintColorStyle_ValueChanged(object sender, EventArgs e)
        {
            var currentItem = (UltraComboEditor)sender;
            ultraGanttViewPrintDocument1.PrintColorStyle = (ColorRenderMode)currentItem.Value;
        }
        #endregion // cmbPrintColorStyle_ValueChanged

        #region chkExpandTasksonPrint_CheckedChanged
        private void chkExpandTasksonPrint_CheckedChanged(object sender, EventArgs e)
        {
            var checkEdit = (UltraCheckEditor)sender;
            ultraGanttViewPrintDocument1.ExpandAllTasks = checkEdit.Checked;
        }
        #endregion // chkExpandTasksonPrint_CheckedChanged

        #region cmbTextCenter_ValueChanged
        private void cmbTextCenter_ValueChanged(object sender, EventArgs e)
        {
            var currentItem = (UltraComboEditor)sender;
            ultraGanttViewPrintDocument1.LegendSettings.DetailsAreaSettings.TextCenter = currentItem.Value.ToString();
        }
        #endregion // cmbTextCenter_ValueChanged

        #region cmbTextRight_ValueChanged
        private void cmbTextRight_ValueChanged(object sender, EventArgs e)
        {
            var currentItem = (UltraComboEditor)sender;
            ultraGanttViewPrintDocument1.LegendSettings.DetailsAreaSettings.TextRight = currentItem.Value.ToString();
        }
        #endregion // cmbTextRight_ValueChanged

        #region cmbTimelineAutoSize_ValueChanged
        private void cmbTimelineAutoSize_ValueChanged(object sender, EventArgs e)
        {
            var currentItem = (UltraComboEditor)sender;
            ultraGanttViewPrintDocument1.AutoSizeTimelineColumns = (AutoSizeTimelineColumns)currentItem.Value;
        }
        #endregion // cmbTimelineAutoSize_ValueChanged

        #endregion // Events

        #region Methods

        #region InitializeForm
        public void InitializeForm()
        {
            // Add the form's caption as the sample name
            ultraFormManager1.FormStyleSettings.Caption = Properties.Resources.SampleTitle;

            // Top and Right Splitter buttons
            this.LoadImagesFor(SplitterTop, SplitterRight);

            // Add the sample description
            sampleInfo.SampleDescription = Resources.SamplesBrowser.SamplesBrowser.SampleDescription;
            sampleInfo.CodeViewerFilePathCS = Properties.Resources.CS_CodeViewerFilePath;
            sampleInfo.CodeViewerFilePathVB = Properties.Resources.VB_CodeViewerFilePath;
            sampleInfo.SampleTitle = Properties.Resources.CodeViewerTitle;

            // Remove focus rectangle.
            ultraTabControl1.DrawFilter = new RemoveFocusRectangle();

            // Create Oeners and project data.
            CreateOwners();
            CreateProjectData();

            // Associate the document printing with GanttView.
            ultraGanttViewPrintDocument1.GanttView = ultraGanttView1;
            ultraPrintPreviewDialog1.Document = ultraGanttViewPrintDocument1;

            cmbShowLegend.DataSource = Enum.GetValues(typeof(ShowLegend));
            cmbShowLegend.SelectedIndex = 1;

            editDetailsAreaWidth.MinValue = 0;
            editDetailsAreaWidth.MaxValue = 5;

            // Initialize detail text in center
            cmbTextCenter.ValueList.ValueListItems.Add(UltraPrintDocument.DatePrintedToken);
            cmbTextCenter.ValueList.ValueListItems.Add(UltraGanttViewPrintDocument.HorizontalPageNumberToken);
            cmbTextCenter.ValueList.ValueListItems.Add(UltraPrintDocument.PageNumberToken);
            cmbTextCenter.ValueList.ValueListItems.Add(UltraPrintDocument.TimePrintedToken);
            cmbTextCenter.ValueList.ValueListItems.Add(UltraPrintDocument.TotalPagesToken);
            cmbTextCenter.ValueList.ValueListItems.Add(UltraPrintDocument.UserNameToken);
            cmbTextCenter.ValueList.ValueListItems.Add(UltraGanttViewPrintDocument.VerticalPageNumberToken);

            // Initialize detail text on right
            cmbTextRight.ValueList.ValueListItems.Add(UltraPrintDocument.DatePrintedToken);
            cmbTextRight.ValueList.ValueListItems.Add(UltraGanttViewPrintDocument.HorizontalPageNumberToken);
            cmbTextRight.ValueList.ValueListItems.Add(UltraPrintDocument.PageNumberToken);
            cmbTextRight.ValueList.ValueListItems.Add(UltraPrintDocument.TimePrintedToken);
            cmbTextRight.ValueList.ValueListItems.Add(UltraPrintDocument.TotalPagesToken);
            cmbTextRight.ValueList.ValueListItems.Add(UltraPrintDocument.UserNameToken);
            cmbTextRight.ValueList.ValueListItems.Add(UltraGanttViewPrintDocument.VerticalPageNumberToken);

            // Initialize option controls 
            cmbTextCenter.SelectedIndex = 0;
            cmbTextRight.SelectedIndex = 1;

            cmbAutoSize.DataSource = Enum.GetValues(typeof(AutoSizeGrid));
            cmbAutoSize.SelectedIndex = 0;

            cmbTimelineAutoSize.DataSource = Enum.GetValues(typeof(AutoSizeTimelineColumns));
            cmbTimelineAutoSize.SelectedIndex = 0;

            cmbPrintColorStyle.DataSource = Enum.GetValues(typeof(ColorRenderMode));
            cmbPrintColorStyle.SelectedIndex = 0;

            cmbLegendAppearances.ValueList.ValueListItems.Add(_rm.GetString("LegendAppearance"));
            cmbLegendAppearances.ValueList.ValueListItems.Add(_rm.GetString("DetailsAreaAppearance"));
            cmbLegendAppearances.ValueList.ValueListItems.Add(_rm.GetString("KeyAreaAppearance"));
            cmbLegendAppearances.SelectedIndex = 0;

            chkExpandTasksonPrint.Checked = true;

            // Initialize the drop-downs for all combo editors to DropDownList.
            cmbAutoSize.DropDownStyle = DropDownStyle.DropDownList;
            cmbLegendAppearances.DropDownStyle = DropDownStyle.DropDownList;
            cmbPrintColorStyle.DropDownStyle = DropDownStyle.DropDownList;
            cmbShowLegend.DropDownStyle = DropDownStyle.DropDownList;
            cmbTextCenter.DropDownStyle = DropDownStyle.DropDownList;
            cmbTextRight.DropDownStyle = DropDownStyle.DropDownList;
            cmbTimelineAutoSize.DropDownStyle = DropDownStyle.DropDownList;
        }
        #endregion // InitializeForm

        #region CreateOwners
        private void CreateOwners()
        {
            ultraCalendarInfo1.Owners.Add("Jon");
            ultraCalendarInfo1.Owners.Add("Jim");
            ultraCalendarInfo1.Owners.Add("Tom");
            ultraCalendarInfo1.Owners.Add("Kim");
            ultraCalendarInfo1.Owners.Add("Ted");
            ultraCalendarInfo1.Owners.Add("Jun");
            ultraCalendarInfo1.Owners.Add("Daniel");
            ultraCalendarInfo1.Owners.Add("Kathey");
            ultraCalendarInfo1.Owners.Add("Jane");
        }
        #endregion

        #region CreateProjectData
        public void CreateProjectData()
        {
            _sampleProject = new Project
            {
                Key = "Project1",
                Name = _rm.GetString("SampleProject"),
                StartDate = DateTime.Today
            };

            // Assign sample project to the projects in calendarInfo object.
            ultraCalendarInfo1.Projects.Add(_sampleProject);
            
            // Assign calendarInfo object to the GanttView control.
            ultraGanttView1.CalendarInfo = ultraCalendarInfo1;
            
            // Assign sample project to the GanttView control.
            ultraGanttView1.Project = ultraGanttView1.CalendarInfo.Projects["Project1"];

            //Add Root Tasks
            Task rootTask = ultraCalendarInfo1.Tasks.Add(DateTime.Today, TimeSpan.FromDays(20), Properties.Resources.RootTask, ultraCalendarInfo1.Projects[1].Key);

            for (int i = 1; i <= 5; i++)
            {
                Task parentTask = rootTask.Tasks.Add(TimeSpan.FromDays(4), String.Format(Properties.Resources.ParentTask, i));

                //Add sub tasks
                Task childTask1 = parentTask.Tasks.Add(DateTime.Today.AddDays(3), TimeSpan.FromDays(5), String.Format(_rm.GetString("ChildTask") + " {0}.1", i));
                
                //Add sub tasks to the above ChildTask1
                Task childTask11 = childTask1.Tasks.Add(DateTime.Today.AddDays(2), TimeSpan.FromDays(1), String.Format(_rm.GetString("ChildTask") + " {0}.1.1", i));
                Task childTask12 = childTask1.Tasks.Add(DateTime.Today.AddDays(5), TimeSpan.FromDays(3), String.Format(_rm.GetString("ChildTask") + " {0}.1.2", i));

                //Add another sub task to the parent task
                Task childTask2 = parentTask.Tasks.Add(DateTime.Today.AddDays(2), TimeSpan.FromDays(5), String.Format(_rm.GetString("ChildTask") + " {0}.2", i));
                
                //Add sub tasks to the above ChildTask1
                Task childTask21 = childTask2.Tasks.Add(DateTime.Today.AddDays(4), TimeSpan.FromDays(1), String.Format(_rm.GetString("ChildTask") + " {0}.2.1", i));
                Task childTask22 = childTask2.Tasks.Add(DateTime.Today.AddDays(1), TimeSpan.FromDays(6), String.Format(_rm.GetString("ChildTask") + " {0}.2.2", i));
                Task childTask23 = childTask2.Tasks.Add(DateTime.Today.AddDays(5), TimeSpan.FromDays(4), String.Format(_rm.GetString("ChildTask") + " {0}.2.3", i));

                //Add another sub task to the Parent task
                Task childTask3 = parentTask.Tasks.Add(DateTime.Today.AddDays(3), TimeSpan.FromDays(3), String.Format(_rm.GetString("ChildTask") + " {0}.3", i));
                
                //Add sub tasks to the above ChildTask1
                Task childTask31 = childTask3.Tasks.Add(DateTime.Today.AddDays(2), TimeSpan.FromDays(2), String.Format(_rm.GetString("ChildTask") + " {0}.3.1", i));
                Task childTask32 = childTask3.Tasks.Add(DateTime.Today.AddDays(4), TimeSpan.FromDays(5), String.Format(_rm.GetString("ChildTask") + " {0}.3.2", i));
                
                //Add Dependencies
                childTask12.Dependencies.Add(childTask11, TaskDependencyType.FinishToStart);
                childTask21.Dependencies.Add(childTask12, TaskDependencyType.StartToStart);
                childTask22.Dependencies.Add(childTask23, TaskDependencyType.FinishToFinish);

                //Add Resources
                childTask1.Resources.Add("Jon");
                childTask11.Resources.Add("Jim");
                childTask12.Resources.Add("Tom");
                childTask2.Resources.Add("Ted");
                childTask21.Resources.Add("Kim");
                childTask22.Resources.Add("Jun");
                childTask23.Resources.Add("Daniel");
                childTask3.Resources.Add("Kathy");
                childTask31.Resources.Add("Jon");
                childTask32.Resources.Add("Jane");

                //Add Milestones
                childTask1.Milestone = true;
                childTask12.Milestone = true;
                childTask32.Milestone = true;

                //Add Percent Complete
                childTask12.PercentComplete = 80;
                childTask21.PercentComplete = 20;
                childTask31.PercentComplete = 90;
                childTask32.PercentComplete = 20;

                //Add Deadline
                childTask22.Deadline = DateTime.Today.AddDays(10);
                childTask11.Deadline = DateTime.Today.AddDays(5);
                childTask21.Deadline = DateTime.Today.AddDays(10);
                childTask32.Deadline = DateTime.Today.AddDays(15);
            }
        }
        #endregion Create Tasks

        #region Legend Appearances
        private AppearanceBase RetrieveAppearanceBasedOnIndex(int index, out string description)
        {
            // Retrieve the appropriate Appearance object (and description) based on the provided index.
            switch (index)
            {
                case 0: // Legend Appearance
                    description = Properties.Resources.DescriptionLegendAppearance;
                    return ultraGanttViewPrintDocument1.LegendSettings.Appearance;

                case 1: // Details Area Appearance
                    description = Properties.Resources.DescriptionDetailsAreaAppearance;
                    return ultraGanttViewPrintDocument1.LegendSettings.DetailsAreaSettings.Appearance;

                default: // Key Area Appearance
                    description = Properties.Resources.DescriptionKeyAreaAppearance;
                    return ultraGanttViewPrintDocument1.LegendSettings.KeyAreaSettings.Appearance;
            }
        }
        #endregion // Legend Appearances

        #endregion // Methods
    }
}
